#!/usr/bin/python
# File: gmpm/__init__.py
# This file is part of yapyb
#
# Copyright (C) 2008 Juan Fiol
# This is free software.  You may redistribute copies of it under the terms of
# the GNU General Public License version 2.
# There is NO WARRANTY, to the extent permitted by law.
#
# Written by Juan Fiol <juanfiol@gmail.com>


__all__ = ["query_ads"]



#!/usr/bin/env python
'''
Script to read an *.aux file generated by latex and extract all the references to a
new bibtex file

'''
import sys
import re
import os

################################################################################
# CUSTOMIZE THESE VARIABLES if needed
dumpfile= os.getenv('BIBDB',os.path.join(os.environ['HOME'],'texmf/bibtex/bib/bib.dmp'))
#*******************************************************************************
encoding='utf8'
import yapbib.biblist as biblist

rex_bibfile= re.compile(r'\\bibliography\{([^}]*)\}')
rex_bibdata= re.compile(r'\\bibdata\{([^}]*)\}')
rex_texcite= re.compile(r'\\(?:no)?cite(?:alt|p|t|author|year)?[\*]?(?:\[[^]]*\]){0,2}\{([^}]*)\}',re.M)
rex_auxcite= re.compile(r'\\citation{([^}]*)}')
rex_auxinput= re.compile(r'\\@input{([^}]+)}')

def get_strng_field(k):
  """Parse arguments of the form string:field"""
  l= unicode(k,encoding=encoding).split(':')
  if len(l) == 1:
    ff= []
    ss= l[0]
  elif len(l) == 2:
    if l[0]=='': ss= '*' # Search all strings
    else:        ss= l[0]
    if l[1]=='':   ff= [] # Search in all fields
    else:          ff= l[1].split(':')
  return ss,ff

def parse_texfile(texfile):
  reflist=[]
  fi= open(texfile);  s= fi.readlines(); fi.close()
  for line in s:
    line= line[:line.find('%')]
    for m in rex_texcite.finditer(line):
      if m!= None:  reflist.extend(m.group(1).split(','))
  return make_unique(reflist)

def parse_auxfile(auxfile): 
  """Parse an auxfile (recursing if necessary) and return citation list"""
  reflist=[]
  fi= open(auxfile);  s= fi.readlines(); fi.close()
  for line in s:
    m1= rex_auxinput.search(line) # Search input of aux file
    if m1: reflist.extend(parse_auxfile(m1.group(1)))
    m= rex_auxcite.search(line)   # Seach for citations
    if m != None:  reflist.append(m.group(1))
  return make_unique(reflist)

def make_unique(lista):
  """Simpler (may fail if lista is not a list) unique function"""
  seen = []
  return list(c for c in lista if not (c in seen or seen.append(c)))

def remove_fields(item, cond):
  ff,ty= get_strng_field(cond)
  if item.get_field('_type') in [t.lower() for t in ty] or ty == []:
    if item.get_field(ff) != None:
      del(item[ff])            

def main():
  import optparse
  usage = """usage: %prog [options] datafile1 [datafile2 ...]
  Extracts a BibTeX database according to an aux or tex file. 
  Keeps only those items that are cited

  DESCRIPTION It reads an *.aux file as produced by LaTeX or a *.tex file directly
       and writes to standard output a bibtex file containing exactly the
       bibtex entries refereed in the aux file.

  NOTE:  If the environment variable BIBDB is set, this is used as bibliography database

  ************************************************************
  USE %prog --help for details
  ************************************************************  
  """
  
  parser = optparse.OptionParser(usage)

  parser.add_option("-d", "--database", action='append', type='string'
                    , help="Database to use, default: %s. May be used more than once" %(dumpfile))

  parser.add_option("-l", "--list", action="store_true", dest="list", default=False
                    , help="List cited keys to stdout (screen)")

  parser.add_option("-o", "--output", default=None
                    , help="Output file. Use '-' for stdout (screen)")

  parser.add_option("", "--remove-common", action="store_true", default=False
                    , help="Remove \"url from articles\", \"doi, issn, month and abstracts from everything\"")

  parser.add_option("", "--remove-fields", action='append', type='string'
                    , help="Remove fields from types. Notations is \"field:type1,type2,..,typen\" to remove field from these types (for instance ARTICLES and BOOKS but not for INPROCEEDINGS), Use \"field\" (with no \":\") for removing the field for all types. It can be used more than once for removing several fields")

  parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False
                    , help="Give some informational messages.")

  (op, args) = parser.parse_args()

  if len(args) < 1:
    parser.error("Incorrect number of arguments")

  if op.database== None:
    dbfiles=[dumpfile]
  else:
    dbfiles=op.database

  # Read the database(s)
  b= biblist.BibList()
  for fname in dbfiles:
    if op.verbose:  print '# Loading database %s ...' %(fname)
    failed=False
    if '.dmp' in fname:
      try:    b.load(fname)
      except: failed=True
    elif '.bib' in fname:
      try: b.import_bibtex(fname, normalize=modify_keys)
      except: failed=True
    else: failed= True

    if failed: 
      mensaje= 'Database file %s not found or failed to load. Set the name as an option or set the environment variable BIBDB'%(fname)
      parser.error(mensaje)

  if op.output == None:  output=os.path.splitext(args[0])[0]+'.bib'
  else:    output= op.output

  # Set fields to remove
  rem=[]
  if op.remove_common:
    rem= rem + ['url:article', 'issn', 'doi', 'month', 'abstract']
  if op.remove_fields != None:
    rem= rem + op.remove_fields

########################################################################
  for fname in args:
    if fname.endswith('tex'):
      cit= parse_texfile(fname)
    elif fname.endswith('aux'):
      cit= parse_auxfile(fname)
    else:
      parser.error('Incorrect argument "%s"'%(fname))

  if op.list:
    print '\n'.join(sorted(cit))

  bout=biblist.BibList()

  for k in cit:
    if k in b.ListItems:
      item= b.get_item(k)
      if rem != []:
        for cond in rem:  remove_fields(item, cond)
      bout.add_item(item, k)
    else:
      print '# Warning: %s not found in database' %(k)
  mensaje='# created with:  %s\n'%(' '.join(sys.argv))
  fi= open(output,'w');  fi.write(mensaje+bout.to_bibtex());  fi.close()
  # bout.output(output,formato='bibtex')
  if op.verbose: print 'Items saved to %s' %(output)


if __name__ == "__main__": main()



### Local Variables: 
### tab-width: 2
### END:



#!/usr/bin/env python
import sys
import os

try:
  from optparse import OptionParser
  import StringIO
  import datetime
except ImportError, e:
  print 'file', __file__ ,e
  sys.exit(1)

try:
  from yapbib import biblist
  import query_ads.query_ads as ads
except ImportError, e:
  print 'Import Error', __file__ ,e
  sys.exit(1)

# usuarios={
#   'Einstein':{'nombre':"Einstein,A.",'mail':'mymail@mydomain.org'},
#   }
# check_strict_list= ['Sanchez',u'S\xe1nchez','Suarez', u'Su\xe1rez']  # Nombres comunes, que solo se agregaran si pasan el siguiente test
# check_strict_cond=[('affiliation', 'Bariloche',True)]

# def check_conditions(item,who,what):
#   """Check conditions on some authors (who) and only keep them if the condition 'what' is true"""
#   autores = [ k[1] for k in item['author']] # Last Name of all authors
  
#   keep_item=True
#   for aut in who:  # A quienes controlamos
#     if aut in autores:  keep_item=False # Tenemos que controlarlos
#   if keep_item:    return True  # No hay que controlarlos

#   for w in what:
#     if item.has_key(w[0]) != w[2]: # No existe el campo requerido (affiliation)
#       return False

#     for aut in who:  # A quienes controlamos
#       try:
#         j=autores.index(aut) 
#         keep_item= False
#       except:
#         continue
#       if (item[w[0]][j].find(w[1]) !=-1 ) == w[2]:
#         return True
#   return keep_item

def main():
  '''Retrieve papers list'''
  # Declara algunas variables
  autores="Einstein,A;Schrodinger,E"
  thisyear= datetime.date.today().year

  #########################################################################
  parser = OptionParser()

  parser.add_option("-o", "--output-file"
                    , help="write to FILE. Default: standard output", metavar="FILE", default='-')
  parser.add_option("-f", "--format", default=None
                    , help="format of output, possible values are: short, full, bibtex, tex, html, xml   DEFAULT= bib")

  parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False
                    , help="Give some informational messages.")

  parser.add_option("-b", "--start-year", default=str(thisyear),
                    help="Starting year as integer (4 digits)")
  parser.add_option("-e", "--end-year", default=str(thisyear),
                    help="Ending year as integer (4 digits)")

  parser.add_option("--year", default= None,
                    help="--year=y is a shortcut to '--start-year=y --end-year=y'")

  parser.add_option("--start-month", default='1',
                    help="Starting month as integer (Jan == 1, Dec == 12)")
  parser.add_option("--end-month", default='12',
                    help="Ending month as integer (Jan == 1, Dec == 12)")
  parser.add_option("--author", default=autores,
                    help="list of semicolon separated author names as last,f")
  parser.add_option("--author-logic", default='OR',
                    help="Logic to use to search for authors. Default: 'OR'. Use 'AND' to get only common (to all authors) articles")
  # parser.add_option("--affiliation", default='Bariloche',
  #                   help="affiliation words, any non-alpha-numeric character separates")
  parser.add_option("--proxy", default=None,
                    help="Proxy used to connect")

  parser.add_option("--advanced-options", default=None,
                    help="""Additional options supported by Harvard. They should be written as option1:value1;option2:value2,...  
                          To get a list of options use: '--help-advanced-options'""")

  parser.add_option("--help-advanced-options", action="store_true", default=False,
                    help="Show information on additional options supported by Harvard ADS site")

  parser.add_option("-d","--save-dump"
                    , help="Save (dump) the database IN INTERNAL FORM")

  parser.add_option("", "--sort", default='key'
                    , help="Sort the items according to the following fields, for instance to sort them accoding to year and then author we would use --sort=year,author. In the same example, to sort in reverse order we would use: --sort=year,author,reverse. DEFAULT: key.")

  (op, args) = parser.parse_args()
  
  if op.help_advanced_options != False:
    if op.verbose:
      print 'Complete list of possible options supported by Harvard:'
      for k,v in ads.all_param.iteritems():
        print '  %18s : %s'%(k,v)
    else:
      print 'The more important parameters are:'
      for k,v in ads.param_relevantes.iteritems():
        print '  %18s : %s'%(k,v)
      print '** To get a complete list use also --verbose **'
    return 1
  
  if op.proxy != None: conexion={'http_proxy': op.proxy}
  else: conexion= {}

  available_formats= {'s':'short','f':'full','t':'latex','b':'bibtex','h':'html','x':'xml'}

  output_file= op.output_file

  if op.format == None:    # Guess a format
    if output_file != '-':
      ext= os.path.splitext(output_file)[1][1]
      if ext in 'tbhx': formato=available_formats.get(ext)
      else: formato='bibtex'
    else: formato='bibtex'
  else:
    formato=available_formats.get(op.format[0].lower())

  #########################################################################################
  opciones={}
  opciones['start_year']=op.start_year
  opciones['end_year']= op.end_year
  if op.year != None:  opciones['start_year']=opciones['end_year']= op.year
    
  opciones['start_mon']=op.start_month
  opciones['end_mon']= op.end_month
  opciones['author']= op.author
  opciones['aut_logic']= op.author_logic
  if op.advanced_options != None:
    for o in op.advanced_options.split(';'):
      k,v= o.split(':')
      opciones[k]= v
  #########################################################################################
  # Create the List object
  b= biblist.BibList()
  #########################################################################################
  Query= ads.AdsQuery(connection=conexion,options=opciones)
  nabst,page= Query.query()
  if nabst < 0:
    print 'Error (%d), %s' %(nabst,page)
    sys.exit()
  else:
    if op.verbose:  print '%d items downloaded' %(nabst)
    
  # Load the results into the biblist object
  fi= StringIO.StringIO(page)
  n= b.import_ads(fi)
  if op.verbose:  print '# %d items downloaded, total number of items %d' %(n, len(b.ListItems))

  sortorder= op.sort.lower().split(',')
  if 'reverse' in sortorder:      reverse=True
  else: reverse= False
  if reverse: sortorder.remove('reverse')
  b.sort(sortorder,reverse=reverse)

  if op.save_dump != None:
    if op.verbose:
      print '# Saving database to %s...' %(op.save_dump)
    b.dump(op.save_dump)

  b.output(output_file, formato, op.verbose)

  


if __name__ == "__main__":
  main()


### Local Variables: 
### tab-width: 2
### END:
